//
//  BXPrinter.h
//  Demo
//
//  Created on 11. 3. 14..
//

#import <Foundation/Foundation.h>
#import "UDPServerDelegate.h"
#import "BXPrinterControlDelegate.h"
#import "InterfaceFunctionsDelegate.h"
//#import "NetFunctionsDelegate.h"
#import "iControllerDelegate.h"
#import "BXCode.h"

//#import "iController.h"

#define __message(fmt, ...)		/*_log(fmt, ##__VA_ARGS__);	\*/\
	if( [self.delegate respondsToSelector:@selector(message:text:)] ) \
		[Common dispatchSelector:@selector(message:text:) \
						  target:self.delegate \
						 objects:[NSArray arrayWithObjects:self,[NSString stringWithFormat:(fmt), ##__VA_ARGS__],nil] \
					onMainThread:YES]

@class	UDPServer;
//@class	NetFunctions;
//@class	BTFunctions;
@class interfaceFunctions;
@class  BXPrinter;
@class	Section;

@interface BXPrinterController : NSObject <UDPServerDelegate, InterfaceFunctionsDelegate, iControllerDelegate>
{
	UDPServer					*_uServer;
	interfaceFunctions			*_ifFunctions;
//	NetFunctions				*_nFunctions;
//	BTFunctions                 *_btFunctions;
	NSTimer						*_lookupTimer;
	NSLock						*_lock;
    NSRecursiveLock				*_lockLookupedArray;
	NSMutableArray				*_lookupedArray;
	
	
	BOOL						_allowLog;

	
	unsigned					_lookupWaitCount;
	
	iController					*_iPrinter;
	
////////////////////////	
	BOOL						_opened;
    BOOL                        _isCheckedMSR;

	Section						*_msrLock;
///**/	Section						*_msrThreadLock;
	
	unsigned char				_msrMode;
	unsigned char				_cardMode;
	
	BOOL						_terminateMsrReadThread;
	unsigned int				_sentinelMode;
	unsigned int				_model;
	unsigned int				_version;
	
	BOOL						_track1Flag;
	BOOL						_track2Flag;
	BOOL						_track3Flag;
	BOOL						_fullTrackFlag;
	
	unsigned char				_track1[MSR_TRACK1];
	unsigned char				_track2[MSR_TRACK2];
	unsigned char				_track3[MSR_TRACK3];
	unsigned char				_fullTrack[MSR_TRACK1 * 3];
	
	BOOL						_encryptedTrackFlag;
	unsigned char				_encryptedTrack[MSR_TRACK_ENCRYPTED];
	
}

@property (retain, readonly)                NSString        *version;          // SDK 버전
@property (retain, readonly)                NSString        *releaseDate;          // SDK 버전
@property (retain, readonly)                NSString        *manufacturer;          // Manufacturer
@property (assign, nonatomic)				int				configPassword;
@property (assign, nonatomic)				int				AutoConnection;
@property (assign, nonatomic)				int				barcodeSupportRange;


@property (assign, nonatomic)				id<BXPrinterControlDelegate>	delegate;
@property (assign, nonatomic)               BXPrinter       *target;

@property (assign, nonatomic)				CGFloat			lookupDuration;		// 프린터 검색 시간
@property (assign, nonatomic, readonly)     unsigned		lookupRemotePort;	// 프린터 검색시 원격 포트 (WIFI/WLAN)
@property (assign, nonatomic, readonly)     unsigned		lookupLocalPort;	// 프린터 검색시 로컬 포트 (WIFI/WLAN)
@property (assign, nonatomic, readonly)     unsigned		lookupRemotePortEthernet;// 프린터 검색시 원격 포트 (Ethernet)
@property (assign, nonatomic, readonly)     unsigned		lookupLocalPortEthernet;// 프린터 검색시 로컬 포트 (Ethernet)
@property (assign, nonatomic)				unsigned		lookupCount;		// 프린터 검색시 브로드캐스팅 횟수


@property (assign, nonatomic)				int				alignment;
@property (assign, nonatomic)				int				attribute;
@property (assign, nonatomic)				int				textSize;
@property (assign, nonatomic)				char			characterSet;       // Codepage
@property (assign, nonatomic)				char			textPosition;   
@property (assign, nonatomic)				char			internationalCharacterSet;  //Codepage Manual Internatinal CharacterSet 내용 참조.
@property (assign, nonatomic)               long            textEncoding;       // iOS 문자열 Encoding 선택.



@property (assign, nonatomic)				int				drawerPin;
@property (assign, nonatomic)				int				drawerOpenLevel;

@property (assign, nonatomic, readonly)     long			state;
@property (assign, nonatomic, readonly)     long			power;

@property (assign, nonatomic)				BOOL			loadConfigurationOnConnect;
@property (assign, nonatomic)				_BXPrinterConfigrationStruct        *config;
@property (assign, nonatomic)				_BXPrinterSettingConfigrationStruct	*settingConfig;
@property (assign, nonatomic)				CGFloat			pendingWaitTime;

+ (BXPrinterController *)getInstance;

- (void)open;
- (void)close;

// 동일 네트워크에 연결된 프린터 검색하기.
- (void)lookup;

// Printer 종류에 따라 프린터 객체 갱신.
- (long) selectTarget;
- (long) selectTarget:(int)modelID;

// 프린터 초기화
- (long)initializePrinter;

// 프린터 상태 정보 얻기.
- (long)checkPrinter;

// 프린터 상태 정보 얻기.
- (long)checkPrinter:(int)mask;


// 프린터와의 TCP 연결하기/연결끊기/연결 확인
- (BOOL)connect;
- (void)disconnect;
- (void)disconnectWithTimeout:(int)timeout;

- (void)disconnectWithTimeout:(int)timeout
                   afterSleep:(int)afterSleep;

- (BOOL)isConnected;

- (int)recvValue:(void *)bytes size:(int)size;

// 텍스트 처리
- (long)printText:(NSString *)string;
- (long)printBox:(int)width height:(int)height;
- (long)lineFeed:(int)lines;
- (long)nextPrintPos;
- (long)cutPaper;

// 바코드 처리
- (long)printBarcode:(char *)bytes
		   symbology:(long)symbology
			   width:(long)width
			  height:(long)height;


- (long)printBarcode:(char *)bytes
           symbology:(long)symbology
               width:(long)width
              height:(long)height
   heightOfSeparator:(long)heightOfSeparator;  //  GS1Databar  Type 확장

// 이미지 처리
- (long)printBitmap:(NSString *)path
			  width:(long)width
			  level:(long)level;

- (long)printBitmapWithImage:(UIImage *)image 
			  width:(long)width 
			  level:(long)level;

// NV Images
- (long)removeNVImage:(int)address;
- (long)removeAllNVImages;
- (long)nvImageList:(NSArray **)images;
- (long)downloadNVImage:(int)address withImage:(UIImage *)image;
- (long)downloadNVImage:(int)address withImage:(UIImage *)image
				  width:(long)width 
				  level:(long)level;
- (long)printNVImage:(int)address;


// MSR 카드 처리
- (long)msrReadReady;
- (long)msrReadCancel;
- (long)msrReadCancelEx;
- (BOOL)msrIsReady;
- (long)msrReadTrack:(NSData **)data1 data2:(NSData **)data2 data3:(NSData **)data3;
- (long)msrGetTrack:(int)track response:(NSData **)response;
- (long)msrReadFullTrack:(NSData **)response;

// IC 카드 처리
- (long)icON:(NSData **)response;
- (long)icOFF;
- (long)icApdu:(NSData *)request response:(NSData **)response;
- (long)icStatus:(NSData **)response;

// Direct IO 처리
- (long)directIO:(NSData *)request 
	requiredSize:(int)requiredSize 
		response:(NSData **)response;

// Cash Drawer 처리
- (long)openDrawer;

// Configuration 
- (long)loadConfiguration;
- (long)saveConfiguration;

// COnfiguration
- (long)loadPrinterConfiguration;
- (long)savePrinterConfiguration;

- (long)enableLSB:(BOOL)bEnable;


// 지원 여부 리턴
- (BOOL) isSupport_Barcode;
- (BOOL) isSupport_MSR;
- (BOOL) isSupport_IC;
- (BOOL) isSupport_Config;
- (BOOL) isSupport_CashDrawer;
- (BOOL) isSupport_LSB;
- (NSMutableArray*)  getBarcodeSupportTable;



// SetObject;

- (void) setConfigPassword:(int)pw;
- (void) setLookupDuration:(CGFloat)duration;
- (void) setLookupCount:(unsigned)count;
- (void) setAlignment:(int)set;
- (void) setAttribute:(int)set;
- (void) setTextSize:(int)size;
- (void) setCharacterSet:(char)set;
- (void) setTextPosition:(char)position;
- (void) setInternationalCharacterSet:(char)set;
- (void) setLoadConfigurationOnConnect:(BOOL)set;
- (void) setPendingWaitTime:(CGFloat)time;
//- (void) setDelegate:(id <BXPrinterControlDelegate>)_delegate;
//- (void) setTarget:(BXPrinter *)_target;

@end
